/*
 *
 * ATM Switch Control Library - swctl
 * 	by miwa@mt.cs.keio.ac.jp, 1997
 * $Id: swctl.h,v 1.1.4.1 1998/09/02 04:02:27 onoe Exp $
 *
 */

#ifndef	_SWCTL_H_
#define	_SWCTL_H_

#include "config.h"

/* swctl global structures */

typedef u_int64_t	swctl_addr_t;	/* ATM Switch VPORT/VPI/VCI */
typedef u_int		switch_id_t;	/* ATM Switch ID */
typedef	u_int		swctl_vport_t;	/* ATM Switch VPORT */
typedef	u_int		swctl_vpi_t;	/* ATM Switch VPI */
typedef	u_int		swctl_vci_t;	/* ATM Switch VCI */

typedef struct _swctl_param_t {	/* QoS parameters per connection */

    u_int pcr01;		/* peak cell rate(cell/sec) for CLP=0,1 */
    u_int scr01;		/* sustainable cell rate(cell/sec) for */
				/* CLP=0,1 */
    u_int mbs01;		/* maximam burstsize(cell/sec) for CLP=0,1 */
    u_int pcr0;			/* peak cell rate(cell/sec) for CLP=0 */
    u_int scr0;			/* sustainable cell rate(cell/sec) for CLP=0 */
    u_int mbs0;			/* maximum burstsize(cell) for CLP=0 */
    u_int cdvt;			/* cell delay variation tolerance (usec) */
    u_int act;			/* action for non-compliant cell: */
				/* 0 = drop, 1 = tag */
    u_int aal5epd;		/* AAL5 EPD: 1 = yes, 0 = no */

} swctl_param_t;

typedef struct _swctl_upc_table_t {

    struct _swctl_upc_table_t	*next_table;	/* linked list */
    int				status;		/* object status */
    int				index;		/* upc index */
    struct _swctl_param_t	*param;		/* QoS parameter */

} swctl_upc_table_t;

typedef struct _swctl_vcc_table_t {

    struct _swctl_vcc_table_t	*next_table;	/* linked list */
    int				status;		/* object status */
    swctl_addr_t		in;		/* in VPORT/VPI/VCI */
    swctl_addr_t		out;		/* out VPORT/VPI/VCI */
    int				upc_index;	/* upc index */
    struct _swctl_param_t	*param;		/* QoS parameter */

} swctl_vcc_table_t;

typedef struct _swctl_switch_attr_t {

    struct _swctl_switch_attr_t	*next;		/* linked list */
    switch_id_t			id;		/* ATM Switch ID */
    u_char			*set_community;
    int				set_community_len;
    u_char			*get_community;
    int				get_community_len;
    struct _swctl_upc_table_t	*upc_table_list;/* upc table list */

} swctl_switch_attr_t;

/* swctl error codes */

#define	SWCTL_SUCCESS		0
#define	SWCTL_ENTRY_FAILURE	-1
#define	SWCTL_LOCAL_FAILURE	-2
#define	SWCTL_SWITCH_FAILURE	-3
#define	SWCTL_VCC_FAILURE	-4
#define	SWCTL_UPC_FAILURE	-5
#define	SWCTL_NETWORK_FAILURE	-6
#define	SWCTL_UNKNOWN_FAILURE	-7

/* swctl_addr_t conversion macros */

#define	ATM2ADDR(vport, vpi, vci) \
((((swctl_addr_t)(vport) & 0xffff) << 32) \
 | (((swctl_addr_t)(vpi) & 0xfff) << 16) \
 | (((swctl_addr_t)(vci) & 0xffff)))

#define	ADDR2VPORT(addr) ((swctl_vport_t)((addr) >> 32))

#define	ADDR2VPI(addr) ((swctl_vpi_t)((addr) & 0xfff0000) >> 16)

#define	ADDR2VCI(addr) ((swctl_vci_t)((addr) & 0xffff))

#define	VPORT2ADDR(vport, addr) \
((swctl_addr_t)(addr) \
 | (((swctl_addr_t)(vport) & 0xffff) << 32)) 

#define	VPI2ADDR(vpi, addr) \
((swctl_addr_t)(addr) \
 | (((swctl_addr_t)(vpi) & 0xfff) << 16))

#define VCI2ADDR(vci, addr) \
((swctl_addr_t)(addr) \
 | ((swctl_addr_t)(vci) & 0xffff)) 

/* swctl api functions */

/* open/close */
int swctl_open __P((switch_id_t sw_id,
		    u_char *set_community,
		    u_char *get_community,
		    swctl_vcc_table_t **vcc_table_list_p));
int swctl_close __P((switch_id_t sw_id));

/* add/delete */
int swctl_add __P((switch_id_t sw_id, swctl_addr_t in, swctl_addr_t out,
		   swctl_param_t *param));
int swctl_delete __P((switch_id_t sw_id, swctl_addr_t in, swctl_addr_t out));

/* modify existing connection's parameter */
int swctl_modify __P((switch_id_t sw_id, swctl_addr_t in, swctl_addr_t out,
		      swctl_param_t *param));

/* redirect existing connection */
int swctl_redirect __P((switch_id_t sw_id, swctl_addr_t in,
			swctl_addr_t out, swctl_addr_t new_out));

/* get connection information */
int swctl_get __P((switch_id_t sw_id, swctl_addr_t in, swctl_addr_t out,
		   swctl_param_t *param));

/* for traffic monioring */
int swctl_mon __P((switch_id_t sw, swctl_addr_t in, swctl_addr_t out,
		   u_int *cells));

/* for switch status */
int swctl_stat __P((switch_id_t sw_id, swctl_vcc_table_t **vcc_table_list_p,
		    swctl_upc_table_t **upc_table_list_p));

/* trap function for swview */
int swview_init __P((u_long addr, u_short port));
int swctl_trap __P((switch_id_t sw));

#endif	/* _SWCTL_H_ */
