/*
 *
 * ATM Switch Control Library - swctl
 *	by miwa@mt.cs.keio.ac.jp, 1997
 * $Id: swctlbulkwalk.c,v 1.1.4.2 1998/09/14 07:08:11 miwa Exp $
 *
 * swctlbulkwalk.c - send SNMPv2 Bulk requests to a network entity, walking a
 * subtree.
 * THIS ORIGINAL SOURCE AS FOLLOWS:
 * snmpbulkwalk.c - send SNMPv2 Bulk requests to a network entity, walking a
 * subtree.
 *
 */
/*********************************************************************
	Copyright 1988, 1989, 1991, 1992 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
**********************************************************************/
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <ctype.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <ucd-snmp/ucd-snmp-config.h>
#include <ucd-snmp/ucd-snmp-includes.h>

#include "config.h"

#include "swctl.h"
#include "swctl_snmp.h"

int swctl_snmp_bulkwalk(struct snmp_session *in_session,
			miboid *root, int rootlen,
			swctl_mib_object_t **mib_object_list_p)
{
    struct snmp_session	session, *ss;
    struct snmp_pdu *pdu, *response;
    struct variable_list *vars;

    swctl_mib_object_t *mib_object;
    swctl_mib_object_t *tail = (swctl_mib_object_t *)NULL;

    miboid	name[MAX_NAME_LEN];
    int name_length;
    int running;
    int status;
    int reps = 1000;
    int	snmp_stat;
    int	return_flag = 0;

#ifdef	DEBUG
    printf("swctl_snmp_bulkwalk()\n");
#endif	/* DEBUG */

    if(mib_object_list_p) {
	return_flag++;
	*(mib_object_list_p) = (swctl_mib_object_t *)NULL;
    }

    /* dirty hack for ucd-snmp-3.6.x or later ... */
    memcpy(&session, in_session, sizeof(session));
    (&session)->community = strdup(in_session->community);
    
    snmp_synch_setup((struct snmp_session *)&session);
    ss = (struct snmp_session *)snmp_open((struct snmp_session
						 *)&session);
    if (ss == NULL)
      return(SNMP_LOCAL_FAILURE);

    memmove(name, root, rootlen*sizeof(miboid));
    name_length = rootlen;

    running = 1;

    while(running){
	running = 1;
	pdu = (struct snmp_pdu *)snmp_pdu_create(SNMP_MSG_GETNEXT);
	pdu->non_repeaters = 0;
	pdu->max_repetitions = reps;	/* fill the packet */

	snmp_add_null_var((struct snmp_pdu *)pdu,
			  (oid *)name, (int)name_length);

	status = snmp_synch_response((struct snmp_session *)ss,
				     (struct snmp_pdu *)pdu,
				     (struct snmp_pdu **)&response);
	if (status == STAT_SUCCESS){
	    if (response->errstat == SNMP_ERR_NOERROR){
		for(vars = response->variables; vars;
		    vars = vars->next_variable){
		    if (vars->name_length < rootlen
			|| memcmp(root, vars->name,
				  rootlen*sizeof(miboid))){
			running = 0;
			continue;	/* not part of this subtree */
		    }

		    if(return_flag) {
			mib_object = (swctl_mib_object_t
				      *)malloc(sizeof(swctl_mib_object_t));
			mib_object->next_object =
			  (swctl_mib_object_t *)NULL;
			copy_variable(vars->name, vars->name_length, vars,
				      mib_object);

			if(!tail)
			  *(mib_object_list_p) = mib_object;
			else 
			  tail->next_object = mib_object;
		    
			tail = mib_object;
		    }

		    if (vars->type == SNMP_ENDOFMIBVIEW
			|| vars->type == SNMP_NOSUCHOBJECT /* for robustness */
			|| vars->type == SNMP_NOSUCHINSTANCE){
			running = 0; /* restart so we can get next variable */
		    }
		    if (!vars->next_variable){ /* repeat on last variable */
			memmove(name, vars->name,
			      vars->name_length*sizeof(miboid));
			name_length = vars->name_length;
		    }
		}
		snmp_stat = SNMP_SUCCESS;
	    }
	    else {
		running = 0;
		snmp_stat = SNMP_PARAM_FAILURE;
	    }

	}
	else if (status == STAT_TIMEOUT){
	    running = 0;
	    snmp_stat = SNMP_TIMEOUT;
	}
	else {    /* status == STAT_ERROR */
	    running = 0;
	    snmp_stat = SNMP_LOCAL_FAILURE;
	}

	if (response)
	  snmp_free_pdu((struct snmp_pdu *)response);
    }
    snmp_close((struct snmp_session *)ss);

    return(snmp_stat);
}
