/*
 *
 * ATM Switch Control Library - sample program
 * 	by miwa@mt.cs.keio.ac.jp, 1997
 * swctlc - scriptable switch controller
 * $Id: swctlc.c,v 1.1.4.1 1998/09/02 04:02:28 onoe Exp $
 *
 */

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <ctype.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

#include "swctl.h"

#define	SWCTLC_SWITCH	0
#define	SWCTLC_OPEN	1
#define	SWCTLC_CLOSE	2
#define	SWCTLC_ADD	3
#define	SWCTLC_DELETE	4
#define	SWCTLC_MODIFY	5
#define	SWCTLC_REDIRECT	6
#define	SWCTLC_GET	7
#define	SWCTLC_STAT	8
#define	SWCTLC_PARAM	9
#define	SWCTLC_NO_PARAM	10
#define	SWCTLC_TIME	11

struct timeval t;

void main( int argc, char **argv, char **envp )
{
    int op, result, i;
    int line = 0;
    int sw_flag = 0;
    int	upc_flag = 0;

    char hoge_buf[BUFSIZ];
    char *hoge_argv[16];
    int	buf_p;
    int	argv_p;

    switch_id_t		sw;
    swctl_vport_t	invport, outvport, new_outvport;
    swctl_vpi_t		invpi, outvpi, new_outvpi;
    swctl_vci_t		invci, outvci, new_outvci;
    swctl_addr_t	in_addr, out_addr, new_out_addr;
    swctl_param_t	*param;
    swctl_vcc_table_t	*vcc_table_list = (swctl_vcc_table_t *)NULL;
    swctl_upc_table_t	*upc_table_list = (swctl_upc_table_t *)NULL;

    param = (swctl_param_t *)malloc(sizeof(swctl_param_t));

    for( ; ; ) {

	/* initialize */
	buf_p = 0;
	argv_p = 0;

	if( fgets(hoge_buf, sizeof(hoge_buf), stdin) == NULL )
	  break;

	line++;
	
	while( hoge_buf[buf_p] != '\n' ) {

	    for( ; hoge_buf[buf_p] == ' ' || hoge_buf[buf_p] == '\t';
		buf_p++);

	    if( hoge_buf[buf_p] == '\n' )
	      break;

	    hoge_argv[argv_p++] = hoge_buf+buf_p;

	    for( ; hoge_buf[buf_p] != ' ' && hoge_buf[buf_p] != '\t'
		&& hoge_buf[buf_p] != '\n'; buf_p++);

	    if (hoge_buf[buf_p] == '\n') {
		hoge_buf[buf_p] = '\0';
		break;
	    }
	    hoge_buf[buf_p++] = '\0';
	}

	if ( argv_p == 0 )		/* space line */
	  continue;

	if( hoge_argv[0][0] == '#')	/* comment line */
	  continue;

	op = str2op(hoge_argv[0]);	/* operation parsing */

	if ( op < 0 ) {			/* unknown opertion */
	    fprintf(stdout, "line: %d: op: %s is not defined.(ignore)\n",
		    line, hoge_argv[0]);
	    continue;
	}

	if ( op > 0 && sw_flag == 0 ) {	/* switch not specified */
	    fprintf(stdout, "line: %d: op: %s ... target switch not specified.(ignore)\n",
		    line, hoge_argv[0]);
	    continue;
	}

	switch(op) {

	  case SWCTLC_SWITCH:
	    if( argv_p < 2 ) {
		fprintf(stdout, "line: %d: op: %s ... too few arguments.(ignore)\n",
			line, hoge_argv[0]);
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }
	    
	    if( sw_flag ) {
		swctl_close(sw);
		sw_flag = 0;
		upc_flag = 0;
	    }
	    
	    sw = (switch_id_t)inet_addr(hoge_argv[1]);
	    if ( SWCTL_SUCCESS == (result = swctl_open(sw,
						       (u_char *)NULL,
						       (u_char *)NULL,
						       &vcc_table_list)))
	      sw_flag = 1;
	    break;

	  case SWCTLC_OPEN:
	    /* obsolete */
	    break;

	  case SWCTLC_CLOSE:
	    /* obsolete */
	    break;

	  case SWCTLC_ADD:
	    if( argv_p < 7 ) {
		fprintf(stdout, "line: %d: op: %s ... too few arguments.(ignore)\n",
			line, hoge_argv[0]);
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }

	    invport = atoi(hoge_argv[1]);
	    invpi = atoi(hoge_argv[2]);
	    invci = atoi(hoge_argv[3]);
	    outvport = atoi(hoge_argv[4]);
	    outvpi = atoi(hoge_argv[5]);
	    outvci = atoi(hoge_argv[6]);
	    
	    in_addr = ATM2ADDR(invport, invpi, invci);
	    out_addr = ATM2ADDR(outvport, outvpi, outvci);

	    if( upc_flag )
	      result = swctl_add(sw, in_addr, out_addr, param);
	    else
	      result = swctl_add(sw, in_addr, out_addr,
				 (swctl_param_t *)NULL);
	    break;

	  case SWCTLC_DELETE:
	    if( argv_p < 7 ) {
		fprintf(stdout, "line: %d: op: %s ... too few arguments.(ignore)\n",
			line, hoge_argv[0]);
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }

	    invport = atoi(hoge_argv[1]);
	    invpi = atoi(hoge_argv[2]);
	    invci = atoi(hoge_argv[3]);
	    outvport = atoi(hoge_argv[4]);
	    outvpi = atoi(hoge_argv[5]);
	    outvci = atoi(hoge_argv[6]);
	    
	    in_addr = ATM2ADDR(invport, invpi, invci);
	    out_addr = ATM2ADDR(outvport, outvpi, outvci);

	    result = swctl_delete(sw, in_addr, out_addr);
	    break;

	  case SWCTLC_MODIFY:
	    if( argv_p < 7 ) {
		fprintf(stdout, "line: %d: op: %s ... too few arguments.(ignore)\n",
			line, hoge_argv[0]);
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }

	    invport = atoi(hoge_argv[1]);
	    invpi = atoi(hoge_argv[2]);
	    invci = atoi(hoge_argv[3]);
	    outvport = atoi(hoge_argv[4]);
	    outvpi = atoi(hoge_argv[5]);
	    outvci = atoi(hoge_argv[6]);
	    
	    in_addr = ATM2ADDR(invport, invpi, invci);
	    out_addr = ATM2ADDR(outvport, outvpi, outvci);

	    if( upc_flag )
	      result = swctl_modify(sw, in_addr, out_addr, param);
	    else
	      result = swctl_modify(sw, in_addr, out_addr,
				    (swctl_param_t *)NULL);
	    break;

	  case SWCTLC_REDIRECT:
	    if( argv_p < 10 ) {
		fprintf(stdout, "line: %d: op: %s ... too few arguments.(ignore)\n",
			line, hoge_argv[0]);
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }

	    invport = atoi(hoge_argv[1]);
	    invpi = atoi(hoge_argv[2]);
	    invci = atoi(hoge_argv[3]);
	    outvport = atoi(hoge_argv[4]);
	    outvpi = atoi(hoge_argv[5]);
	    outvci = atoi(hoge_argv[6]);
	    new_outvport = atoi(hoge_argv[7]);
	    new_outvpi = atoi(hoge_argv[8]);
	    new_outvci = atoi(hoge_argv[9]);
	    
	    in_addr = ATM2ADDR(invport, invpi, invci);
	    out_addr = ATM2ADDR(outvport, outvpi, outvci);
	    new_out_addr = ATM2ADDR(new_outvport, new_outvpi, new_outvci);

	    result = swctl_redirect(sw, in_addr, out_addr, new_out_addr);
	    break;

	  case SWCTLC_GET:
	    /* obsolete */
	    break;
	    
	  case SWCTLC_STAT:
	    result = swctl_stat(sw, &vcc_table_list, &upc_table_list);
	    print_vcc_table_list(vcc_table_list);
	    free_vcc_table_list(vcc_table_list);
	    vcc_table_list = (swctl_vcc_table_t *)NULL;
	    print_upc_table_list(upc_table_list);
	    free_upc_table_list(upc_table_list);
	    upc_table_list = (swctl_upc_table_t *)NULL;
	    break;

	  case SWCTLC_PARAM:
	    if( argv_p < 10 ) {
		fprintf(stdout, "line: %d: op: %s ... too few arguments.(ignore)\n",
			line, hoge_argv[0]);
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }

	    memset((void *)param, 0, sizeof(swctl_param_t));

	    param->pcr01 = atoi(hoge_argv[1]);
	    param->scr01 = atoi(hoge_argv[2]);
	    param->mbs01 = atoi(hoge_argv[3]);
	    param->pcr0 = atoi(hoge_argv[4]);
	    param->scr0 = atoi(hoge_argv[5]);
	    param->mbs0 = atoi(hoge_argv[6]);
	    param->cdvt = atoi(hoge_argv[7]);
	    param->act = atoi(hoge_argv[8]);
	    param->aal5epd = atoi(hoge_argv[9]);

	    result = SWCTL_SUCCESS;
	    upc_flag = 1;
		
	    break;

	  case SWCTLC_NO_PARAM:
	    upc_flag = 0;
	    result = SWCTL_SUCCESS;

	    break;

	  case SWCTLC_TIME:
	    gettimeofday(&t, (void *)NULL);
	    fprintf(stdout, "%s\t%u usec\n", ctime(&t.tv_sec), t.tv_usec);
	    result = SWCTL_SUCCESS;

	  default:
	    break;
				
	}

	for(i = 0; i < argv_p; i++)
	  fprintf(stdout, "%s ", hoge_argv[i]);

	switch(result) {
	  case SWCTL_SUCCESS:
	    fprintf(stdout, "...OK");
	    break;

	  case SWCTL_ENTRY_FAILURE:
	    fprintf(stdout, "...Serious Failure(perhaps bug...)");
	    break;
	    
	  case SWCTL_LOCAL_FAILURE:
	    fprintf(stdout, "...Serious Failure(perhaps bug...)");
	    break;

	  case SWCTL_SWITCH_FAILURE:
	    fprintf(stdout, "...Switch SNMP Failure(switch SNMP Agent broken?).");
	    break;
	    
	  case SWCTL_VCC_FAILURE:
	    fprintf(stdout, "...This VCC already established or not established yet.");
	    break;
	    
	  case SWCTL_UPC_FAILURE:
	    fprintf(stdout, "...UPC failed(cannot assign enough resource).");
	    break;

	  case SWCTL_NETWORK_FAILURE:
	    fprintf(stdout, "...Network error(wrong IP address or network down?).");
	    break;
	    
	  default:
	    break;
	}

	fprintf(stdout, "\n");
    }
    exit(0);
}

int str2op(char *str)
{
    /* match prefix 3 letters --> yes */

    if(!memcmp((void *)str, "switch", 3*sizeof(char)))
      return(SWCTLC_SWITCH);

    if(!memcmp((void *)str, "open", 3*sizeof(char)))
      return(SWCTLC_OPEN);
    
    if(!memcmp((void *)str, "close", 3*sizeof(char)))
      return(SWCTLC_CLOSE);

    if(!memcmp((void *)str, "add", 3*sizeof(char)))
      return(SWCTLC_ADD);

    if(!memcmp((void *)str, "delete", 3*sizeof(char)))
      return(SWCTLC_DELETE);

    if(!memcmp((void *)str, "modify", 3*sizeof(char)))
      return(SWCTLC_MODIFY);
    
    if(!memcmp((void *)str, "redirect", 3*sizeof(char)))
      return(SWCTLC_REDIRECT);
    
    if(!memcmp((void *)str, "get", 3*sizeof(char)))
      return(SWCTLC_GET);

    if(!memcmp((void *)str, "stat", 3*sizeof(char)))
      return(SWCTLC_STAT);

    if(!memcmp((void *)str, "param", 3*sizeof(char)))
      return(SWCTLC_PARAM);

    if(!memcmp((void *)str, "noparam", 3*sizeof(char)))
      return(SWCTLC_NO_PARAM);

    if(!memcmp((void *)str, "time", 3*sizeof(char)))
      return(SWCTLC_TIME);

    return(-1);		/* default */
}

int print_param(swctl_param_t *param)
{
    fprintf(stdout,
            "PCR01\tSCR01\tMBS01\tPCR0\tSCR0\tMBS0\tCDVT\tACT\tAAL5EPD\n");

    if(param) {

        fprintf(stdout, "%u\t", param->pcr01);
        fprintf(stdout, "%u\t", param->scr01);
        fprintf(stdout, "%u\t", param->mbs01);
        fprintf(stdout, "%u\t", param->pcr0);
        fprintf(stdout, "%u\t", param->scr0);
        fprintf(stdout, "%u\t", param->mbs0);
        fprintf(stdout, "%u\t", param->cdvt);
        if(param->act == 0) 
          fprintf(stdout, "tag\t");
        else
          fprintf(stdout, "drop\t");
        if(param->aal5epd == 0) 
          fprintf(stdout, "no\n");
        else
          fprintf(stdout, "yes\n");
    }
}
