/*
 *
 * ATM Switch Control Library - swctl
 *	by miwa@mt.cs.keio.ac.jp, 1997
 * $Id: swctlset.c,v 1.1.4.2 1998/09/14 07:08:14 miwa Exp $
 *
 * swctlset.c - send snmp SET requests to a network entity.
 * THIS ORIGINAL SOURCE AS FOLLOWS:
 * snmpset.c - send snmp SET requests to a network entity.
 *
 */
/***********************************************************************
	Copyright 1988, 1989, 1991, 1992 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <ctype.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <ucd-snmp/ucd-snmp-config.h>
#include <ucd-snmp/ucd-snmp-includes.h>

#include "config.h"

#include "swctl.h"
#include "swctl_snmp.h"

int
swctl_snmp_set(struct snmp_session *in_session,
	       swctl_mib_object_t *mib_object_list,
	       swctl_mib_object_t **mib_object_list_p)
{
    struct snmp_session session, *ss;
    struct snmp_pdu *pdu, *response;
    struct variable_list *vars;

    swctl_mib_object_t *mib_object;
    swctl_mib_object_t *tail = (swctl_mib_object_t *)NULL;

    int status;
    int snmp_stat;
    int return_flag = 0;

#ifdef	DEBUG
    printf("swctl_snmp_set()\n");
#endif	/* DEBUG */

    if(mib_object_list_p) {
	return_flag++;
	*(mib_object_list_p) = (swctl_mib_object_t *)NULL;
    }

    /* dirty hack for ucd-snmp-3.6.x or later ... */
    memcpy(&session, in_session, sizeof(session));
    (&session)->community = strdup(in_session->community);

    snmp_synch_setup((struct snmp_session *)&session);
    ss = (struct snmp_session *)snmp_open((struct snmp_session
						 *)&session);
    if (ss == NULL)
      return(SNMP_LOCAL_FAILURE);

    pdu = (struct snmp_pdu *)snmp_pdu_create(SNMP_MSG_SET);

    for( mib_object = mib_object_list; mib_object; mib_object =
	mib_object->next_object )
      snmp_add_var_int(pdu, mib_object->name, mib_object->name_length,
		       mib_object->value);

    status = snmp_synch_response((struct snmp_session *)ss,
				 (struct snmp_pdu *)pdu,
				 (struct snmp_pdu **)&response);

    if (status == STAT_SUCCESS){
	if (response->errstat == SNMP_ERR_NOERROR){
	    for(vars = response->variables; vars; vars =
		vars->next_variable) {
		if(return_flag) {
		    mib_object = (swctl_mib_object_t
				  *)malloc(sizeof(swctl_mib_object_t));
		    mib_object->next_object =
		      (swctl_mib_object_t *)NULL;
		    copy_variable(vars->name, vars->name_length, vars,
				  mib_object);

		    if(!tail)
		      *(mib_object_list_p) = mib_object;
		    else 
		      tail->next_object = mib_object;
		
		    tail = mib_object;
		}
	    }
	    
	    snmp_stat = SNMP_SUCCESS;
	}
	else {
	    snmp_stat = SNMP_PARAM_FAILURE;
	}

    }
    else if (status == STAT_TIMEOUT){
	snmp_stat = SNMP_TIMEOUT;
    }
    else {    /* status == STAT_ERROR */
	snmp_stat = SNMP_LOCAL_FAILURE;
    }

    if (response)
      snmp_free_pdu((struct snmp_pdu *)response);
    snmp_close((struct snmp_session *)ss);

    return(snmp_stat);
}

/*
 * Add a variable with the requested name to the end of the list of
 * variables for this pdu.(modified for integer)
 */
int
snmp_add_var_int(struct snmp_pdu *pdu, miboid *name, int name_length,
		 int value)
{
    struct variable_list *vars;
    char buf[2048];

#ifdef	DEBUG
    printf("swctl_add_var_int()\n");
#endif	/* DEBUG */

    if (pdu->variables == NULL){
	pdu->variables = vars =
	    (struct variable_list *)malloc(sizeof(struct variable_list));
    } else {
	for(vars = pdu->variables;
	    vars->next_variable;
	    vars = vars->next_variable)
	    /*EXIT*/;
	vars->next_variable =
	    (struct variable_list *)malloc(sizeof(struct variable_list));
	vars = vars->next_variable;
    }

    vars->next_variable = NULL;
    vars->name = (miboid *)malloc(name_length * sizeof(miboid));
    memmove(vars->name, name, name_length * sizeof(miboid));
    vars->name_length = name_length;

    vars->type = ASN_INTEGER;
    vars->val.integer = (long *)malloc(sizeof(long));
    *(vars->val.integer) = (long)value;
    vars->val_len = sizeof(long);
}

int
swctl_snmp_set_gauge(struct snmp_session *session,
		     swctl_mib_object_t *mib_object_list,
		     swctl_mib_object_t **mib_object_list_p)
{
    struct snmp_session *ss;
    struct snmp_pdu *pdu, *response;
    struct variable_list *vars;

    swctl_mib_object_t *mib_object;
    swctl_mib_object_t *tail = (swctl_mib_object_t *)NULL;

    int status;
    int snmp_stat;
    int return_flag = 0;

#ifdef	DEBUG
    printf("swctl_snmp_set_gauge()\n");
#endif	/* DEBUG */

    if(mib_object_list_p) {
	return_flag++;
	*(mib_object_list_p) = (swctl_mib_object_t *)NULL;
    }

    snmp_synch_setup((struct snmp_session *)session);
    ss = (struct snmp_session *)snmp_open((struct snmp_session
						 *)session); 
    if (ss == NULL)
      return(SNMP_LOCAL_FAILURE);

    pdu = (struct snmp_pdu *)snmp_pdu_create(SNMP_MSG_SET);

    for( mib_object = mib_object_list; mib_object; mib_object =
	mib_object->next_object )
      snmp_add_var_gauge(pdu, mib_object->name, mib_object->name_length,
			 mib_object->value);

    status = snmp_synch_response((struct snmp_session *)ss,
				 (struct snmp_pdu *)pdu,
				 (struct snmp_pdu **)&response);

    if (status == STAT_SUCCESS){
	if (response->errstat == SNMP_ERR_NOERROR){
	    for(vars = response->variables; vars; vars =
		vars->next_variable) {
		if(return_flag) {
		    mib_object = (swctl_mib_object_t
				  *)malloc(sizeof(swctl_mib_object_t));
		    mib_object->next_object =
		      (swctl_mib_object_t *)NULL;
		    copy_variable(vars->name, vars->name_length, vars,
				  mib_object);

		    if(!tail)
		      *(mib_object_list_p) = mib_object;
		    else 
		      tail->next_object = mib_object;
		
		    tail = mib_object;
		}
	    }
	    
	    snmp_stat = SNMP_SUCCESS;
	}
	else {
	    snmp_stat = SNMP_PARAM_FAILURE;
	}

    }
    else if (status == STAT_TIMEOUT){
	snmp_stat = SNMP_TIMEOUT;
    }
    else {    /* status == STAT_ERROR */
	snmp_stat = SNMP_LOCAL_FAILURE;
    }

    if (response)
      snmp_free_pdu((struct snmp_pdu *)response);
    snmp_close((struct snmp_session *)ss);

    return(snmp_stat);
}

/*
 * Add a variable with the requested name to the end of the list of
 * variables for this pdu.(modified for guage)
 */
int snmp_add_var_gauge(struct snmp_pdu *pdu, miboid *name,
		       int name_length, u_long value)
{
    struct variable_list *vars;
    char buf[2048];

#ifdef	DEBUG
    printf("swctl_add_var_gauge()\n");
#endif	/* DEBUG */

    if (pdu->variables == NULL){
	pdu->variables = vars =
	    (struct variable_list *)malloc(sizeof(struct variable_list));
    } else {
	for(vars = pdu->variables;
	    vars->next_variable;
	    vars = vars->next_variable)
	    /*EXIT*/;
	vars->next_variable =
	    (struct variable_list *)malloc(sizeof(struct variable_list));
	vars = vars->next_variable;
    }

    vars->next_variable = NULL;
    vars->name = (miboid *)malloc(name_length * sizeof(miboid));
    memmove(vars->name, name, name_length * sizeof(miboid));
    vars->name_length = name_length;

    vars->type = ASN_GAUGE;
    vars->val.integer = (u_long *)malloc(sizeof(long));
    *(vars->val.integer) = (u_long)value;
    vars->val_len = sizeof(u_long);
}
