/*
 *
 * ATM Switch Control Library - sample program
 * 	by miwa@mt.cs.keio.ac.jp, 1997
 * swctlsh - switch control shell
 * $Id: swctlsh.c,v 1.1.4.1 1998/09/02 04:02:30 onoe Exp $
 *
 */
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <ctype.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

#include "swctl.h"

void main( int argc, char **argv, char **envp )
{
    void *ptr;
    int op, result;
    u_int cells;

    struct hostent	hostentry;
    struct in_addr	in;
    switch_id_t		sw;
    swctl_vport_t	vport;
    swctl_vpi_t		vpi;
    swctl_vci_t		vci;
    swctl_addr_t	in_addr, out_addr, new_out_addr;
    swctl_param_t	*param;
    swctl_vcc_table_t	*vcc_table_list = (swctl_vcc_table_t *)NULL;
    swctl_upc_table_t	*upc_table_list = (swctl_upc_table_t *)NULL;

    param = (swctl_param_t *)malloc(sizeof(swctl_param_t));

    if ( argc < 2 ) {
	fprintf(stderr, "swctlsh: Usage: swctlsh <switch name>\n");
	exit(1);
    }
    
    if( (struct hostent *)NULL ==
       (ptr = (struct hostent *)gethostbyname(argv[1]))) {
	fprintf(stderr, "swctlsh: Unknown switch: %s.\n", argv[1]);
	exit(1);
    }

    hostentry = *(struct hostent *)ptr;  
    in = *((struct in_addr *)(hostentry.h_addr));
    memcpy((void *)&sw, (void *)&in, sizeof(switch_id_t));
    
    fprintf(stderr, "Trying %s ... ", inet_ntoa(in));
    if( SWCTL_SUCCESS != swctl_open(sw, (u_char *)NULL, (u_char *)NULL,
				    &vcc_table_list) ) {
	fprintf(stderr, "switch open failure.(exit)\n");
	exit(-1);
    }
    fprintf(stderr, "connect.\n");

    for( ; ; ) {

	fprintf(stderr, "*** swctlsh Operations ***\n");
	fprintf(stderr, " 1. swctl_add\n");
	fprintf(stderr, " 2. swctl_delete\n");
	fprintf(stderr, " 3. swctl_modify\n");
	fprintf(stderr, " 4. swctl_redirect\n");
	fprintf(stderr, " 5. swctl_stat\n");
	fprintf(stderr, " 0. exit\n");

	fprintf(stderr, "input OP no. --> ");
	if( EOF == scanf("%d", &op) ) {
	    fprintf(stderr, "exit.\n");
	    exit(0);
	}

	switch(op) {
	  case 1:
	    fprintf(stderr, "input in vport vpi vci --> ");
	    if( EOF == scanf("%u %u %u", &vport, &vpi, &vci) ) {
		fprintf(stderr, "break!\n");
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }
	    in_addr = ATM2ADDR(vport, vpi, vci);

	    fprintf(stderr, "input out vport vpi vci --> ");
	    if( EOF == scanf("%u %u %u", &vport, &vpi, &vci) ) {
		fprintf(stderr, "break!\n");
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }
	    
	    out_addr = ATM2ADDR(vport, vpi, vci);

	    fprintf(stderr, "do you want to set param ? (y or n) ");
	    getchar();

	    if( getchar() == 'y' ) {
	        input_param(param);
	        result = swctl_add(sw, in_addr, out_addr, param);
	    }
	    else {
	        result = swctl_add(sw, in_addr, out_addr,
				   (swctl_param_t *)NULL);
	    }
	    break;
	    
	  case 2:
	    
	    fprintf(stderr, "input in vport vpi vci --> ");
	    if( EOF == scanf("%u %u %u", &vport, &vpi, &vci) ) {
		fprintf(stderr, "break!\n");
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }

	    in_addr = ATM2ADDR(vport, vpi, vci);

	    fprintf(stderr, "input out vport vpi vci --> ");
	    if( EOF == scanf("%u %u %u", &vport, &vpi, &vci) ) {
		fprintf(stderr, "break!\n");
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }
		  
	    out_addr = ATM2ADDR(vport, vpi, vci);

	    result = swctl_delete(sw, in_addr, out_addr);
	    break;

	  case 3:
	    fprintf(stderr, "input in vport vpi vci --> ");
	    if( EOF == scanf("%u %u %u", &vport, &vpi, &vci) ) {
		fprintf(stderr, "break!\n");
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }
	    in_addr = ATM2ADDR(vport, vpi, vci);

	    fprintf(stderr, "input out vport vpi vci --> ");
	    if( EOF == scanf("%u %u %u", &vport, &vpi, &vci) ) {
		fprintf(stderr, "break!\n");
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }
	    out_addr = ATM2ADDR(vport, vpi, vci);

	    input_param(param);

	    result = swctl_modify(sw, in_addr, out_addr, param);
	    break;

	  case 4:
	    fprintf(stderr, "input in vport vpi vci --> ");
	    if( EOF == scanf("%u %u %u", &vport, &vpi, &vci) ) {
		fprintf(stderr, "break!\n");
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }
	    in_addr = ATM2ADDR(vport, vpi, vci);

	    fprintf(stderr, "input out vport vpi vci --> ");
	    if( EOF == scanf("%u %u %u", &vport, &vpi, &vci) ) {
		fprintf(stderr, "break!\n");
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }
	    out_addr = ATM2ADDR(vport, vpi, vci);

	    fprintf(stderr, "new input out vport vpi vci --> ");
	    if( EOF == scanf("%u %u %u", &vport, &vpi, &vci) ) {
		fprintf(stderr, "break!\n");
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }
	    new_out_addr = ATM2ADDR(vport, vpi, vci);

	    result = swctl_redirect(sw, in_addr, out_addr, new_out_addr);
	    break;

	  case 5:
	    result = swctl_stat(sw, &vcc_table_list, &upc_table_list);
	    fprintf(stderr, "\n***** switch %s vcc list *****\n", argv[1]);
	    print_vcc_table_list(vcc_table_list);
	    free_vcc_table_list(vcc_table_list);
	    vcc_table_list = (swctl_vcc_table_t *)NULL;
	    fprintf(stderr, "\n***** switch %s upc list *****\n", argv[1]);
	    print_upc_table_list(upc_table_list);
	    free_upc_table_list(upc_table_list);
	    upc_table_list = (swctl_upc_table_t *)NULL;
	    break;

	  case 6:
	    fprintf(stderr, "input in vport vpi vci --> ");
	    if( EOF == scanf("%u %u %u", &vport, &vpi, &vci) ) {
		fprintf(stderr, "break!\n");
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }
	    in_addr = ATM2ADDR(vport, vpi, vci);
	    
	    fprintf(stderr, "input out vport vpi vci --> ");
	    if( EOF == scanf("%u %u %u", &vport, &vpi, &vci) ) {
		fprintf(stderr, "break!\n");
		result = SWCTL_UNKNOWN_FAILURE;
		break;
	    }
	    out_addr = ATM2ADDR(vport, vpi, vci);

	    result = swctl_mon(sw, in_addr, out_addr, &cells);
	    fprintf(stderr, "%u cells\n", cells);
	    break;

	  case 0:
	    swctl_close(sw);
	    exit(0);

	  default:
	    break;
	}
	
	fprintf(stderr, "result --> ");

	switch(result) {
	  case SWCTL_SUCCESS:
	    fprintf(stderr, "SWCTL_SUCCESS");
	    break;

	  case SWCTL_ENTRY_FAILURE:
	    fprintf(stderr, "SWCTL_ENTRY_FAILURE");
	    break;
	    
	  case SWCTL_LOCAL_FAILURE:
	    fprintf(stderr, "SWCTL_LOCAL_FAILURE");
	    break;

	  case SWCTL_SWITCH_FAILURE:
	    fprintf(stderr, "SWCTL_SWITCH_FAILURE");
	    break;
	    
	  case SWCTL_VCC_FAILURE:
	    fprintf(stderr, "SWCTL_VCC_FAILURE");
	    break;
	    
	  case SWCTL_UPC_FAILURE:
	    fprintf(stderr, "SWCTL_UPC_FAILURE");
	    break;

	  case SWCTL_NETWORK_FAILURE:
	    fprintf(stderr, "SWCTL_NETWORK_FAILURE");
	    break;

	  default:
	    fprintf(stderr, "BREAK!");
	    break;
	}
	fprintf(stderr, "\n");

    }
    /* NOTREACHED */
}

int input_param(swctl_param_t *param)
{
    u_int parameter;

    memset((void *)param, 0, sizeof(swctl_param_t));

    if(param) {

	fprintf(stderr, "input pcr01(cell/sec) --> ");
	scanf("%u", &parameter);
	param->pcr01 = parameter;

	fprintf(stderr, "input scr01(cell/sec) --> ");
	scanf("%u", &parameter);
	param->scr01 = parameter;

	fprintf(stderr, "input mbs01(cell/sec) --> ");
	scanf("%u", &parameter);
	param->mbs01 = parameter;
    
	fprintf(stderr, "input pcr0(cell/sec) --> ");
	scanf("%u", &parameter);
	param->pcr0 = parameter;

	fprintf(stderr, "input scr0(cell/sec) --> ");
	scanf("%u", &parameter);
	param->scr0 = parameter;

	fprintf(stderr, "input mbs0(cell/sec) --> ");
	scanf("%u", &parameter);
	param->mbs0 = parameter;
    
	fprintf(stderr, "input cdvt(usec) --> ");
	scanf("%u", &parameter);
	param->cdvt = parameter;

	fprintf(stderr, "input act(0=drop;1=tag) --> ");
	scanf("%u", &parameter);
	param->act = parameter;

	fprintf(stderr, "input aal5epd(0=no;1=yes) --> ");
	scanf("%u", &parameter);
	param->aal5epd = parameter;
    }
}

int print_param(swctl_param_t *param)
{
    fprintf(stderr,
            "PCR01\tSCR01\tMBS01\tPCR0\tSCR0\tMBS0\tCDVT\tACT\tAAL5EPD\n");

    if(param) {

        fprintf(stderr, "%u\t", param->pcr01);
        fprintf(stderr, "%u\t", param->scr01);
        fprintf(stderr, "%u\t", param->mbs01);
        fprintf(stderr, "%u\t", param->pcr0);
        fprintf(stderr, "%u\t", param->scr0);
        fprintf(stderr, "%u\t", param->mbs0);
        fprintf(stderr, "%u\t", param->cdvt);
        if(param->act == 0) 
          fprintf(stderr, "tag\t");
        else
          fprintf(stderr, "drop\t");
        if(param->aal5epd == 0) 
          fprintf(stderr, "no\n");
        else
          fprintf(stderr, "yes\n");
    }
}
