/*
 *
 * ATM Switch Control Library - swctl
 * 	by miwa@mt.cs.keio.ac.jp, 1997
 * $Id: swctltool.c,v 1.1.4.3 1999/04/28 10:16:51 kyu Exp $
 *
 */

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <ctype.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"

#include <ucd-snmp/ucd-snmp-config.h>
#include <ucd-snmp/ucd-snmp-includes.h>

#include "swctl.h"
#include "swctl_snmp.h"

int
copy_variable(miboid *obj_name, int name_length, struct variable_list
	      *variable, swctl_mib_object_t *mib_object)
{
    miboid *name;

    name = (miboid *)malloc(name_length*sizeof(miboid));
    memcpy((void *)name, (void *)obj_name, name_length*sizeof(miboid));

    mib_object->name = name;
    mib_object->name_length = name_length;

    if(variable->type == ASN_INTEGER)
      mib_object->value = *(variable->val.integer);

    return(0);
} 

int
clean_junk_object(struct snmp_session *session, swctl_mib_object_t 
		  *mib_object_list)
{
    swctl_mib_object_t *mib_object;
    swctl_mib_object_t *mib_object_tmp;
    swctl_mib_object_t *mib_object_tmp_list = (swctl_mib_object_t
					       *)NULL;
    swctl_mib_object_t *mib_object_tmp2_list = (swctl_mib_object_t
					       *)NULL;
    
    for( mib_object = mib_object_list; mib_object;
	mib_object = mib_object->next_object ) {

	if(mib_object->value != SNMP_VALID_VAL) {

	    mib_object_tmp = (swctl_mib_object_t
			      *)malloc(sizeof(swctl_mib_object_t));
	    memcpy((void *)mib_object_tmp, (void *)mib_object,
		   sizeof(swctl_mib_object_t));
	    mib_object_tmp->next_object = mib_object_tmp_list;
	    mib_object_tmp->value = SNMP_INVALID_VAL;
	}
    }
    
    swctl_snmp_set(session, mib_object_tmp_list, &mib_object_tmp2_list);

    free_mib_object_list(mib_object_tmp_list);
    free_mib_object_list(mib_object_tmp2_list);

    return(0);
}

swctl_mib_object_t *
make_mib_object(miboid *prefix, int prefix_len, miboid*	suffix,
		int suffix_len, int value)
{
    swctl_mib_object_t *mib_object;
    
    mib_object = (swctl_mib_object_t
		  *)malloc(sizeof(swctl_mib_object_t));

    mib_object->next_object = (swctl_mib_object_t *)NULL;

    if(prefix_len == 0 && suffix_len == 0)
      mib_object->name = (miboid*)NULL;
    else
      mib_object->name =
	(miboid*)malloc((prefix_len+suffix_len)*sizeof(miboid));

    if(prefix_len > 0)
      memcpy((void *)(mib_object->name), (void *)prefix,
	     prefix_len*sizeof(miboid));
    if(suffix_len > 0)
      memcpy((void *)(mib_object->name+prefix_len), (void *)suffix,
	     suffix_len*sizeof(miboid));

    mib_object->name_length = prefix_len+suffix_len;
    mib_object->value = value;

    return(mib_object);
}

int
object_prefix_cmp(miboid *src_name, int src_name_length,
		  miboid *cmp_name, int cmp_name_length)
{
    if(src_name == (miboid*)NULL)
      return(0);
    
    if(src_name_length < cmp_name_length
       || memcmp((void *)src_name, (void *)cmp_name,
		 cmp_name_length*sizeof(miboid)) )
      return(0);

    /* exactly match src_name prefix and cmp_name */
    return(1);		/* match */
}

int
free_mib_object_list(swctl_mib_object_t *mib_object_list)
{
    swctl_mib_object_t	*current;
    swctl_mib_object_t	*previous;

    for( current = mib_object_list, previous = (swctl_mib_object_t
						*)NULL;
	current;
	previous = current, current = current->next_object,
	free(previous) )
      if(current->name)
	free(current->name);
    
    return(0);
}
