/*
 *
 * ATM Switch Control Library - swctl
 * 	by miwa@mt.cs.keio.ac.jp, 1997
 * $Id: swctlvcc.c,v 1.1.4.1 1998/09/02 04:02:31 onoe Exp $
 *
 */

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <ctype.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <ucd-snmp/ucd-snmp-config.h>
#include <ucd-snmp/ucd-snmp-includes.h>

#include "config.h"

#include "swctl.h"
#include "swctl_snmp.h"

swctl_mib_object_t *
make_vcc_rt_object(miboid *prefix, int prefix_len, swctl_addr_t addr1,
		swctl_addr_t addr2, int value)
{
    miboid suffix[6];
    swctl_mib_object_t *mib_object;

    suffix[0] = ADDR2VPORT(addr1);
    suffix[1] = ADDR2VPI(addr1);
    suffix[2] = ADDR2VCI(addr1);
    suffix[3] = ADDR2VPORT(addr2);
    suffix[4] = ADDR2VPI(addr2);
    suffix[5] = ADDR2VCI(addr2);

    mib_object = make_mib_object(prefix, prefix_len, suffix, 6, value);

    return(mib_object);
}

swctl_mib_object_t *
make_vcc_object(miboid *prefix, int prefix_len, swctl_addr_t addr,
		int value)
{
    miboid suffix[6];
    swctl_mib_object_t *mib_object;

    suffix[0] = ADDR2VPORT(addr);
    suffix[1] = ADDR2VPI(addr);
    suffix[2] = ADDR2VCI(addr);

    mib_object = make_mib_object(prefix, prefix_len, suffix, 3, value);

    return(mib_object);
}

int
print_vcc_table(swctl_vcc_table_t *vcc_table)
{
    if(vcc_table) {
	fprintf(stdout, "%u\t", ADDR2VPORT(vcc_table->in));
	fprintf(stdout, "%u\t", ADDR2VPI(vcc_table->in));
	fprintf(stdout, "%u\t", ADDR2VCI(vcc_table->in));
	fprintf(stdout, "%u\t", ADDR2VPORT(vcc_table->out));
	fprintf(stdout, "%u\t", ADDR2VPI(vcc_table->out));
	fprintf(stdout, "%u\t", ADDR2VCI(vcc_table->out));
	if(vcc_table->param)
	  fprintf(stdout, "yes(%d)\n", vcc_table->upc_index);
	else
	  fprintf(stdout, "no(%d)\n", vcc_table->upc_index);
    }	  
    return(0);
}

int
print_vcc_table_list(swctl_vcc_table_t *vcc_table_list)
{
    swctl_vcc_table_t *vcc_table;

    fprintf(stdout,
	    "INPORT\tINVPI\tINVCI\tOUTPORT\tOUTVPI\tOUTVCI\tUPC\n");

    for( vcc_table = vcc_table_list; vcc_table; vcc_table =
	vcc_table->next_table )
      print_vcc_table(vcc_table);

    return(0);
}

int
free_vcc_table_list(swctl_vcc_table_t *vcc_table_list)
{
    swctl_vcc_table_t *current;
    swctl_vcc_table_t *previous;

    for( current = vcc_table_list, previous = (swctl_vcc_table_t *)NULL;
	current;
	previous = current, current = current->next_table,
	free(previous) ) {
	if(current->param)
	  free(current->param);
    }

    return(0);
}
